import Gio from 'gi://Gio';
import GObject from 'gi://GObject';

import * as Main from 'resource:///org/gnome/shell/ui/main.js';

import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
import {QuickToggle, SystemIndicator} from 'resource:///org/gnome/shell/ui/quickSettings.js';

const LightModeToggle = GObject.registerClass(
class LightModeToggle extends QuickToggle {
    _init() {
        super._init({
            title: _('Light Style'),
            iconName: 'weather-clear-symbolic',
        });

        this._settings = new Gio.Settings({
            schema_id: 'org.gnome.desktop.interface',
        });
        this._changedId = this._settings.connect('changed::color-scheme',
            () => this._sync());
            
        this.connectObject(
            'destroy', () => this._destroy(),
            'clicked', () => this._toggleMode(),
            this);
        this._sync();
    }

    _destroy() {
      this._settings = null;
      this._settings.disconnect(this._changedId);
      this.disconnectObject(this);
    }

    _toggleMode() {
        Main.layoutManager.screenTransition.run();
        this._settings.set_string('color-scheme',
            this.checked ? 'default' : 'prefer-light');
    }

    _sync() {
        const colorScheme = this._settings.get_string('color-scheme');
        const checked = colorScheme === 'prefer-light';
        if (this.checked !== checked)
            this.set({checked});
    }    
});

const LuminusIndicator = GObject.registerClass(
class LuminusIndicator extends SystemIndicator {
    _init() {
        super._init();

        this.quickSettingsItems.push(new LightModeToggle());
    }
});

export default class LuminusExtension extends Extension {

    syncEnable() {        
        const colorScheme = this._settings.get_string('color-scheme');
        const isDark = colorScheme === 'prefer-dark';
        if(isDark) {
            Main.layoutManager.screenTransition.run();
            this._settings.set_string('color-scheme', 'prefer-dark');
        } else {
            Main.layoutManager.screenTransition.run();
            this._settings.set_string('color-scheme', 'prefer-light');
          };
        }

    syncDisable() {        
        const colorScheme = this._settings.get_string('color-scheme');
        const isDark = colorScheme === 'prefer-dark';
        if(isDark){
            Main.layoutManager.screenTransition.run();
            this._settings.set_string('color-scheme', 'prefer-dark');
        } else {
            Main.layoutManager.screenTransition.run();
            this._settings.set_string('color-scheme', 'default');
          };
        } 

    enable() {   
        this._settings = new Gio.Settings({schema_id: 'org.gnome.desktop.interface',});    
        this.syncEnable();
        
        this._indicator = new LuminusIndicator();
        Main.panel.statusArea.quickSettings.addExternalIndicator(this._indicator);
    }
    
    disable() {
        this.syncDisable();
        this._settings = null;

        this._indicator.quickSettingsItems.forEach(item => item.destroy());
        this._indicator.destroy();
        this._indicator = null;
    }  
}
